import time
import random

sortable = """
<script>
/*
  SortTable
  version 2
  7th April 2007
  Stuart Langridge, http://www.kryogenix.org/code/browser/sorttable/
    
  Thanks to many, many people for contributions and suggestions.
  Licenced as X11: http://www.kryogenix.org/code/browser/licence.html
  This basically means: do what you want with it.
*/

 
var stIsIE = /*@cc_on!@*/false;

sorttable = {
  init: function() {
    // quit if this function has already been called
    if (arguments.callee.done) return;
    // flag this function so we don't do the same thing twice
    arguments.callee.done = true;
    // kill the timer
    if (_timer) clearInterval(_timer);
    
    if (!document.createElement || !document.getElementsByTagName) return;
    
    sorttable.DATE_RE = /^(\\d\\d?)[\\/\\.-](\\d\\d?)[\\/\\.-]((\\d\\d)?\\d\\d)$/;
    
    forEach(document.getElementsByTagName('table'), function(table) {
      if (table.className.search(/\\bsortable\\b/) != -1) {
        sorttable.makeSortable(table);
      }
    });
    
  },
  
  makeSortable: function(table) {
    if (table.getElementsByTagName('thead').length == 0) {
      // table doesn't have a tHead. Since it should have, create one and
      // put the first table row in it.
      the = document.createElement('thead');
      the.appendChild(table.rows[0]);
      table.insertBefore(the,table.firstChild);
    }
    // Safari doesn't support table.tHead, sigh
    if (table.tHead == null) table.tHead = table.getElementsByTagName('thead')[0];
    
    if (table.tHead.rows.length != 1) return; // can't cope with two header rows
    
    // Sorttable v1 put rows with a class of "sortbottom" at the bottom (as
    // "total" rows, for example). This is B&R, since what you're supposed
    // to do is put them in a tfoot. So, if there are sortbottom rows,
    // for backwards compatibility, move them to tfoot (creating it if needed).
    sortbottomrows = [];
    for (var i=0; i<table.rows.length; i++) {
      if (table.rows[i].className.search(/\\bsortbottom\\b/) != -1) {
        sortbottomrows[sortbottomrows.length] = table.rows[i];
      }
    }
    if (sortbottomrows) {
      if (table.tFoot == null) {
        // table doesn't have a tfoot. Create one.
        tfo = document.createElement('tfoot');
        table.appendChild(tfo);
      }
      for (var i=0; i<sortbottomrows.length; i++) {
        tfo.appendChild(sortbottomrows[i]);
      }
      delete sortbottomrows;
    }
    
    // work through each column and calculate its type
    headrow = table.tHead.rows[0].cells;
    for (var i=0; i<headrow.length; i++) {
      // manually override the type with a sorttable_type attribute
      if (!headrow[i].className.match(/\\bsorttable_nosort\\b/)) { // skip this col
        mtch = headrow[i].className.match(/\\bsorttable_([a-z0-9]+)\\b/);
        if (mtch) { override = mtch[1]; }
	      if (mtch && typeof sorttable["sort_"+override] == 'function') {
	        headrow[i].sorttable_sortfunction = sorttable["sort_"+override];
	      } else {
	        headrow[i].sorttable_sortfunction = sorttable.guessType(table,i);
	      }
	      // make it clickable to sort
	      headrow[i].sorttable_columnindex = i;
	      headrow[i].sorttable_tbody = table.tBodies[0];
	      dean_addEvent(headrow[i],"click", function(e) {

          if (this.className.search(/\\bsorttable_sorted\\b/) != -1) {
            // if we're already sorted by this column, just 
            // reverse the table, which is quicker
            sorttable.reverse(this.sorttable_tbody);
            this.className = this.className.replace('sorttable_sorted',
                                                    'sorttable_sorted_reverse');
            this.removeChild(document.getElementById('sorttable_sortfwdind'));
            sortrevind = document.createElement('span');
            sortrevind.id = "sorttable_sortrevind";
            sortrevind.innerHTML = stIsIE ? '&nbsp<font face="webdings">5</font>' : '&nbsp;&#x25B4;';
            this.appendChild(sortrevind);
            return;
          }
          if (this.className.search(/\\bsorttable_sorted_reverse\\b/) != -1) {
            // if we're already sorted by this column in reverse, just 
            // re-reverse the table, which is quicker
            sorttable.reverse(this.sorttable_tbody);
            this.className = this.className.replace('sorttable_sorted_reverse',
                                                    'sorttable_sorted');
            this.removeChild(document.getElementById('sorttable_sortrevind'));
            sortfwdind = document.createElement('span');
            sortfwdind.id = "sorttable_sortfwdind";
            sortfwdind.innerHTML = stIsIE ? '&nbsp<font face="webdings">6</font>' : '&nbsp;&#x25BE;';
            this.appendChild(sortfwdind);
            return;
          }
          
          // remove sorttable_sorted classes
          theadrow = this.parentNode;
          forEach(theadrow.childNodes, function(cell) {
            if (cell.nodeType == 1) { // an element
              cell.className = cell.className.replace('sorttable_sorted_reverse','');
              cell.className = cell.className.replace('sorttable_sorted','');
            }
          });
          sortfwdind = document.getElementById('sorttable_sortfwdind');
          if (sortfwdind) { sortfwdind.parentNode.removeChild(sortfwdind); }
          sortrevind = document.getElementById('sorttable_sortrevind');
          if (sortrevind) { sortrevind.parentNode.removeChild(sortrevind); }
          
          this.className += ' sorttable_sorted';
          sortfwdind = document.createElement('span');
          sortfwdind.id = "sorttable_sortfwdind";
          sortfwdind.innerHTML = stIsIE ? '&nbsp<font face="webdings">6</font>' : '&nbsp;&#x25BE;';
          this.appendChild(sortfwdind);

	        // build an array to sort. This is a Schwartzian transform thing,
	        // i.e., we "decorate" each row with the actual sort key,
	        // sort based on the sort keys, and then put the rows back in order
	        // which is a lot faster because you only do getInnerText once per row
	        row_array = [];
	        col = this.sorttable_columnindex;
	        rows = this.sorttable_tbody.rows;
	        for (var j=0; j<rows.length; j++) {
	          row_array[row_array.length] = [sorttable.getInnerText(rows[j].cells[col]), rows[j]];
	        }
	        /* If you want a stable sort, uncomment the following line */
	        //sorttable.shaker_sort(row_array, this.sorttable_sortfunction);
	        /* and comment out this one */
	        row_array.sort(this.sorttable_sortfunction);
	        
	        tb = this.sorttable_tbody;
	        for (var j=0; j<row_array.length; j++) {
	          tb.appendChild(row_array[j][1]);
	        }
	        
	        delete row_array;
	      });
	    }
    }
  },
  
  guessType: function(table, column) {
    // guess the type of a column based on its first non-blank row
    sortfn = sorttable.sort_alpha;
    for (var i=0; i<table.tBodies[0].rows.length; i++) {
      text = sorttable.getInnerText(table.tBodies[0].rows[i].cells[column]);
      if (text != '') {
        if (text.match(/^-?[$$$]?[\\d,.]+%?$/)) {
          return sorttable.sort_numeric;
        }
        // check for a date: dd/mm/yyyy or dd/mm/yy 
        // can have / or . or - as separator
        // can be mm/dd as well
        possdate = text.match(sorttable.DATE_RE)
        if (possdate) {
          // looks like a date
          first = parseInt(possdate[1]);
          second = parseInt(possdate[2]);
          if (first > 12) {
            // definitely dd/mm
            return sorttable.sort_ddmm;
          } else if (second > 12) {
            return sorttable.sort_mmdd;
          } else {
            // looks like a date, but we can't tell which, so assume
            // that it's dd/mm (English imperialism!) and keep looking
            sortfn = sorttable.sort_ddmm;
          }
        }
      }
    }
    return sortfn;
  },
  
  getInnerText: function(node) {
    // gets the text we want to use for sorting for a cell.
    // strips leading and trailing whitespace.
    // this is *not* a generic getInnerText function; it's special to sorttable.
    // for example, you can override the cell text with a customkey attribute.
    // it also gets .value for <input> fields.
    
    hasInputs = (typeof node.getElementsByTagName == 'function') &&
                 node.getElementsByTagName('input').length;
    
    if (node.getAttribute("sorttable_customkey") != null) {
      return node.getAttribute("sorttable_customkey");
    }
    else if (typeof node.textContent != 'undefined' && !hasInputs) {
      return node.textContent.replace(/^\\s+|\\s+$/g, '');
    }
    else if (typeof node.innerText != 'undefined' && !hasInputs) {
      return node.innerText.replace(/^\\s+|\\s+$/g, '');
    }
    else if (typeof node.text != 'undefined' && !hasInputs) {
      return node.text.replace(/^\\s+|\\s+$/g, '');
    }
    else {
      switch (node.nodeType) {
        case 3:
          if (node.nodeName.toLowerCase() == 'input') {
            return node.value.replace(/^\\s+|\\s+$/g, '');
          }
        case 4:
          return node.nodeValue.replace(/^\\s+|\\s+$/g, '');
          break;
        case 1:
        case 11:
          var innerText = '';
          for (var i = 0; i < node.childNodes.length; i++) {
            innerText += sorttable.getInnerText(node.childNodes[i]);
          }
          return innerText.replace(/^\\s+|\\s+$/g, '');
          break;
        default:
          return '';
      }
    }
  },
  
  reverse: function(tbody) {
    // reverse the rows in a tbody
    newrows = [];
    for (var i=0; i<tbody.rows.length; i++) {
      newrows[newrows.length] = tbody.rows[i];
    }
    for (var i=newrows.length-1; i>=0; i--) {
       tbody.appendChild(newrows[i]);
    }
    delete newrows;
  },
  
  /* sort functions
     each sort function takes two parameters, a and b
     you are comparing a[0] and b[0] */
  sort_numeric: function(a,b) {
    aa = parseFloat(a[0].replace(/[^0-9.-]/g,''));
    if (isNaN(aa)) aa = 0;
    bb = parseFloat(b[0].replace(/[^0-9.-]/g,'')); 
    if (isNaN(bb)) bb = 0;
    return aa-bb;
  },
  sort_alpha: function(a,b) {
    if (a[0]==b[0]) return 0;
    if (a[0]<b[0]) return -1;
    return 1;
  },
  sort_ddmm: function(a,b) {
    mtch = a[0].match(sorttable.DATE_RE);
    y = mtch[3]; m = mtch[2]; d = mtch[1];
    if (m.length == 1) m = '0'+m;
    if (d.length == 1) d = '0'+d;
    dt1 = y+m+d;
    mtch = b[0].match(sorttable.DATE_RE);
    y = mtch[3]; m = mtch[2]; d = mtch[1];
    if (m.length == 1) m = '0'+m;
    if (d.length == 1) d = '0'+d;
    dt2 = y+m+d;
    if (dt1==dt2) return 0;
    if (dt1<dt2) return -1;
    return 1;
  },
  sort_mmdd: function(a,b) {
    mtch = a[0].match(sorttable.DATE_RE);
    y = mtch[3]; d = mtch[2]; m = mtch[1];
    if (m.length == 1) m = '0'+m;
    if (d.length == 1) d = '0'+d;
    dt1 = y+m+d;
    mtch = b[0].match(sorttable.DATE_RE);
    y = mtch[3]; d = mtch[2]; m = mtch[1];
    if (m.length == 1) m = '0'+m;
    if (d.length == 1) d = '0'+d;
    dt2 = y+m+d;
    if (dt1==dt2) return 0;
    if (dt1<dt2) return -1;
    return 1;
  },
  
  shaker_sort: function(list, comp_func) {
    // A stable sort function to allow multi-level sorting of data
    // see: http://en.wikipedia.org/wiki/Cocktail_sort
    // thanks to Joseph Nahmias
    var b = 0;
    var t = list.length - 1;
    var swap = true;

    while(swap) {
        swap = false;
        for(var i = b; i < t; ++i) {
            if ( comp_func(list[i], list[i+1]) > 0 ) {
                var q = list[i]; list[i] = list[i+1]; list[i+1] = q;
                swap = true;
            }
        } // for
        t--;

        if (!swap) break;

        for(var i = t; i > b; --i) {
            if ( comp_func(list[i], list[i-1]) < 0 ) {
                var q = list[i]; list[i] = list[i-1]; list[i-1] = q;
                swap = true;
            }
        } // for
        b++;

    } // while(swap)
  }  
}

/* ******************************************************************
   Supporting functions: bundled here to avoid depending on a library
   ****************************************************************** */

// Dean Edwards/Matthias Miller/John Resig

/* for Mozilla/Opera9 */
if (document.addEventListener) {
    document.addEventListener("DOMContentLoaded", sorttable.init, false);
}

/* for Internet Explorer */
/*@cc_on @*/
/*@if (@_win32)
    document.write("<script id=__ie_onload defer src=javascript:void(0)><\\/script>");
    var script = document.getElementById("__ie_onload");
    script.onreadystatechange = function() {
        if (this.readyState == "complete") {
            sorttable.init(); // call the onload handler
        }
    };
/*@end @*/

/* for Safari */
if (/WebKit/i.test(navigator.userAgent)) { // sniff
    var _timer = setInterval(function() {
        if (/loaded|complete/.test(document.readyState)) {
            sorttable.init(); // call the onload handler
        }
    }, 10);
}

/* for other browsers */
window.onload = sorttable.init;

// written by Dean Edwards, 2005
// with input from Tino Zijdel, Matthias Miller, Diego Perini

// http://dean.edwards.name/weblog/2005/10/add-event/

function dean_addEvent(element, type, handler) {
	if (element.addEventListener) {
		element.addEventListener(type, handler, false);
	} else {
		// assign each event handler a unique ID
		if (!handler.$$guid) handler.$$guid = dean_addEvent.guid++;
		// create a hash table of event types for the element
		if (!element.events) element.events = {};
		// create a hash table of event handlers for each element/event pair
		var handlers = element.events[type];
		if (!handlers) {
			handlers = element.events[type] = {};
			// store the existing event handler (if there is one)
			if (element["on" + type]) {
				handlers[0] = element["on" + type];
			}
		}
		// store the event handler in the hash table
		handlers[handler.$$guid] = handler;
		// assign a global event handler to do all the work
		element["on" + type] = handleEvent;
	}
};
// a counter used to create unique IDs
dean_addEvent.guid = 1;

function removeEvent(element, type, handler) {
	if (element.removeEventListener) {
		element.removeEventListener(type, handler, false);
	} else {
		// delete the event handler from the hash table
		if (element.events && element.events[type]) {
			delete element.events[type][handler.$$guid];
		}
	}
};

function handleEvent(event) {
	var returnValue = true;
	// grab the event object (IE uses a global event object)
	event = event || fixEvent(((this.ownerDocument || this.document || this).parentWindow || window).event);
	// get a reference to the hash table of event handlers
	var handlers = this.events[event.type];
	// execute each event handler
	for (var i in handlers) {
		this.$$handleEvent = handlers[i];
		if (this.$$handleEvent(event) === false) {
			returnValue = false;
		}
	}
	return returnValue;
};

function fixEvent(event) {
	// add W3C standard event methods
	event.preventDefault = fixEvent.preventDefault;
	event.stopPropagation = fixEvent.stopPropagation;
	return event;
};
fixEvent.preventDefault = function() {
	this.returnValue = false;
};
fixEvent.stopPropagation = function() {
  this.cancelBubble = true;
}

// Dean's forEach: http://dean.edwards.name/base/forEach.js
/*
	forEach, version 1.0
	Copyright 2006, Dean Edwards
	License: http://www.opensource.org/licenses/mit-license.php
*/

// array-like enumeration
if (!Array.forEach) { // mozilla already supports this
	Array.forEach = function(array, block, context) {
		for (var i = 0; i < array.length; i++) {
			block.call(context, array[i], i, array);
		}
	};
}

// generic enumeration
Function.prototype.forEach = function(object, block, context) {
	for (var key in object) {
		if (typeof this.prototype[key] == "undefined") {
			block.call(context, object[key], key, object);
		}
	}
};

// character enumeration
String.forEach = function(string, block, context) {
	Array.forEach(string.split(""), function(chr, index) {
		block.call(context, chr, index, string);
	});
};

// globally resolve forEach enumeration
var forEach = function(object, block, context) {
	if (object) {
		var resolve = Object; // default
		if (object instanceof Function) {
			// functions have a "length" property
			resolve = Function;
		} else if (object.forEach instanceof Function) {
			// the object implements a custom forEach method so use that
			object.forEach(block, context);
			return;
		} else if (typeof object == "string") {
			// the object is a string
			resolve = String;
		} else if (typeof object.length == "number") {
			// the object is array-like
			resolve = Array;
		}
		resolve.forEach(object, block, context);
	}
};

</script>

<style type="text/css">
body{font-family:"Lucida Sans Unicode", "Lucida Grande", Verdana, Arial, Helvetica, sans-serif; font-size:12px; color:#555555;}
div.main{margin:30px auto; width:1000px;}
table.sortable{border:0; padding:0; margin:0;}
table.sortable td{padding:4px; width:120px; border-bottom:solid 1px #DEDEDE;}
table.sortable th{padding:4px;}
table.sortable thead{background:#ccffcc; color:#333333; text-align:left;}
table.sortable tfoot{font-weight:bold; }
table.sortable tfoot td{border:none;}
</style>

"""

import string

class Shuffleboard:
    "returns bogus hexadecimal valuesfor values 0 through 254 inclusive"
    def __init__(self):
        self.vals = []
        for i in range(0,255):
            self.vals.append(i)

        random.shuffle(self.vals)

    def sub(self,plaintext):
        try:
            plaintext = plaintext.strip()
            plaintext = int(plaintext)
        except:
            return "XX"

        if (plaintext < 0 or plaintext > 254):
            return "XX"
        return "%X" % (self.vals[plaintext])

    def fudge_ip(self, ip, fudgelevel):
        "replaces the first <fudgelevel> octets of a dotted decimal IP with consistent gibberish"

        #comment following line to enable fudging.  (for examples)
        return ip
        #fudgelevel = 4
        
        octets = ip.split(".")
    
        if (fudgelevel > 0):
            octets[0] = self.sub(octets[0])
        if (fudgelevel > 1):
            octets[1] = self.sub(octets[1])
        if (fudgelevel > 2):
            octets[2] = self.sub(octets[2])
        if (fudgelevel > 3):
            octets[3] = self.sub(octets[3])

        ip = string.join(octets,".")

        return ip


sb = Shuffleboard()    


class Connection:
    def __init__(self, data, bitsdelta,time):
        self.data = data

        self.bits = data.split()[8]
        self.flags = data.split()[10]

        self.bitsdelta = bitsdelta
        self.time = time

    def output(self):
        global sb
        #print " " * 10, "outputting Connection"
        #print " "*10, self.bits, self.flags
        #print "%s bits over %s time for %s <br>" % (self.bitsdelta, self.time, self.data)

        print "<tr>"

        print "<td>%s</td>" % (self.bitsdelta)
        print "<td>%.02f</td>" % (self.bitsdelta / self.time)
        print "<td>%.02f</td>" % (self.bitsdelta / self.time / 1024)
        #print "<td>%s</td>" % (self.bitsdelta / self.time / 1024 / 1024)

        data = self.data.split()
        #sb.fudge_ip("192.168.100.1", 3)
        #print "<td>%s</td>" % data[2]
        print "<td>%s</td>" % sb.fudge_ip(data[2], 2)
        print "<td>%s</td>" % data[2].split(":")[1]
        #print "<td>%s</td>" % data[4]
        print "<td>%s</td>" % sb.fudge_ip(data[4], 2)
        print "<td>%s</td>" % data[4].split(":")[1]
        print "<td>%s</td>" % data[6]

        print "<td>%s</td>" % data[8]
        print "<td>%s</td>" % data[10]        
        

        print "</tr>"


    def getbits(self):
        return self.bits

    def getflags(self):
        return self.flags

    def describe(self):
        return self.data

class Talker:
    def __init__(self):
        self.connections = {}


    def add_connection(self, local_port, remote_ip, remote_port, string, bitsdelta, time):
        self.connections[(local_port, remote_ip, remote_port)] = Connection(string,bitsdelta,time)

    def output(self):
        #print ".....outputting Talker"
        for key,val in enumerate(self.connections):
            self.connections[val].output()
    
        

class TimeSlice:
    def __init__(self,when=0):
        self.talkers = {}

        if when == 0:
            self.time = time.time()
        else:
            self.time = when

    def add_connection(self, local_ip, local_port, remote_ip, remote_port, string, bitsdelta=0,time=0):

        try:
            if self.talkers[local_ip] == "anything at all":
                pass
        except:
            self.talkers[local_ip] = Talker()

        self.talkers[local_ip].add_connection(local_port, remote_ip, remote_port, string, bitsdelta, time)

    def output(self):
        #print "outputting TimeSlice"
        

        for key,val in enumerate(self.talkers):
        #for key,val in self.talkers:
            #print key,val
            #print self.talkers[val]
            self.talkers[val].output()


    def get_time(self):
        return self.time

    


def timeslice_from_data(data):
    t = TimeSlice()
    #count = 0
    for each in data.split('\n'):
        #count += 1
        #print count, each[:3]

        if each[:3] == "TCP":
            parts = each.split()
            remotepair = parts[2]
            localpair= parts[4]
            idle = parts[6]
            bits = parts[8]
            flags = parts[10]

            #print remotepair,localpair,idle,bits,flags

            t.add_connection( localpair.split(":")[0], localpair.split(":")[1],  remotepair.split(":")[0], remotepair.split(":")[1], each)
            #print each
            
            
        elif each[:3] == "UDP":
            pass
        else:
            pass
    return t


def timeslice_from_diff(curr_t, prev_t):

    diff = TimeSlice()
    
    for key,val in enumerate(curr_t.talkers):
        #print key,val
        #curr_t.talkers[val]

        for key2,val2 in enumerate(curr_t.talkers[val].connections):

            newbits =  int(curr_t.talkers[val].connections[val2].bits.replace(',',''))
            try:
                oldbits =  int(prev_t.talkers[val].connections[val2].bits.replace(',',''))
            except:
                oldbits = 0

            #print newbits,oldbits
                    
            if newbits == oldbits:
                #print "MATCHY", curr_t.talkers[val].connections[val2].describe()
                pass

            if newbits > oldbits:

                each =  curr_t.talkers[val].connections[val2].describe()
                #print newbits - oldbits, "new bits for",  curr_t.talkers[val].connections[val2].describe()
                
                parts = each.split()
                remotepair = parts[2]
                localpair= parts[4]
                idle = parts[6]
                bits = parts[8]
                flags = parts[10]
    
                diff.add_connection( localpair.split(":")[0], localpair.split(":")[1],  remotepair.split(":")[0], remotepair.split(":")[1], each, newbits-oldbits, curr_t.time - prev_t.time)


            if oldbits > newbits:
                pass
                #print "I don't think this should ever happen"
            #print key2,val2

    return diff


def conn_grep(text):
    buf = ""
    #for each in text.split("\n"):
    for each in text:
        #if each[:3] in ("TCP", "UDP", "ICM", "GRE"):
        if each[:3] in ("TCP", "UDP"):
            buf = "%s\n%s" % ( buf, each)

    #print buf
    return buf



def print_header():
    print """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
    <html>
        <head>
                <title>Connflow</title>
"""
#                <link rel="stylesheet" href="style.css" type="text/css" />
#
#
#                <script type='text/javascript' src='common.js'></script>
#                <script type='text/javascript' src='css.js'></script>

#                <script type='text/javascript' src='standardista-table-sorting.js'></script>
    print sortable

    print """
        </head>
        <body>
    """

    print """
    <div class='main'>
    <p>The following output is generated from a diff of two "show conn" commands in a Cisco Pix (or pix-like) firewall.  The time between the "show conn" commands is somewhat approximate due to  network lag and such, so the 's" value in such things as "kbps" may be imprecise and should be taken with salt.</p>
    <p>For comparison, the raw output of "show conn" is displayed below the chart.  Currently only TCP is considered.</p>

    <p>Further - a connection that existed in the first scrape, but not the second, contributes zero bits per second to this chart.  A connection that exists in the second scrape but not the first is treated as if has existed for the full polling period.  A connection that is created and torn down between the polling periods will not show at all.  These are the limitations of only having two samples.</p>

    </div>


"""

def print_table_header():

    print "<div class='main'>"
    print "<table class='sortable' cellspacing='0'>"
    print "<thead>"
    print "<tr>"
    print "<th>new bits</th>"
    print "<th>bps</th>"
    print "<th>kbps</th>"
    print "<th>foreign ip:port</th>"
    print "<th>foreign port</th>"
    print "<th>local ip:port</th>"
    print "<th>local port</th>"
    print "<th>idle</th>"
    print "<th>total bits</th>"
    print "<th>flags</th>"
    print "</tr>"
    print "</thead>"
    print "<tbody>"

def print_table_footer():
    print "</tbody>"
    #print "<tfoot>"
    #print "<tr>"
    #print "<th> </th>"
    #print "<th> </th>"
    #print "<th> </th>"
    #print "<th> </th>"
    #print "<th> </th>"
    #print "<th> </th>"
    #print "<th> </th>"
    #print "<th> </th>"
    #print "<th> </th>"
    #print "<th> </th>"
    #print "</tr>"
    #print "</tfoot>"
    print "</table>"
    print "</div>"


def print_footer():
    print "<div class='main'>"
    print "<br><font size=small>Generated by connflow.py by <a href='http://www.elifulkerson.com'>Eli Fulkerson</a>, written in Python and using Plink (part of Simon Tatham's thoroughly excellent PuTTY) for the ssh remoting.</font>"
    print "<br><font size=small>Sortable table javascript from - Stuart Langridge, http://www.kryogenix.org/code/browser/sorttable/, which further included add_event code from Dean Edwards, 2005, with input from Tino Zijdel, Matthias Miller, Diego Perini (http://dean.edwards.name/weblog/2005/10/add-event/)</font>"
    print "</div>"
    print "</body>"
    print "</html>"

def print_sortable_raw( output ):
    print "<div class='main'>"
    print "<table class='nonsortable' cellspacing='0'>"
    #print "<h3>First Scrape (raw output, from approx %s)</h3>" % (time.ctime(t1.get_time()))
    #print "<pre>"

    most = 0
    for each in output:
        current = 0
        for each in each.split(' '):
            current = current + 1
        most = max(current, most)

    print "<thead>"
    print "<tr>",
    print "<th>line #</th>",
    for i in range(0, most):
        print "<th></th>",
    print "</tr>"
    print "</thead>"

    print "<tbody>"
    count = 0
    for each in output:
        print "<tr>",
        print "<td>%s</td>" % (count)
        for word in each.split(' '):
            print "<td>%s</td>" % (word),

        for i in range(1, most - len(each.split(' '))):
            print "<td></td>",
        print "</tr>"
        count = count + 1

    print "</tbody>"
    print "</table>"
    print "</div>"
    
    


import pix_scrape
import time

buffer = ""
for each in time.localtime()[:6]:
    buffer = "%s_%s" % (buffer, str(each))
filename = "connflow%s.html" % (buffer)
rawfilename = buffer


info_header = ""

targetip = raw_input("Target IP address:")
user = raw_input("Username (blank for default (pix)):")
linepw = raw_input("Line Password:")
enableuser = raw_input("Enable User (blank for default (none)):")
enablepw = raw_input("Enable Password:")
sshversion = raw_input("ssh version (1,2) (default 2):")

info_header = "<div class='main'>Report for cisco device at: %s <br>  Current time is: %s <br> Interval between checks is approximately ten seconds. </div>" % (targetip, time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.gmtime()))

if user == "":
    user = "pix"

if enableuser == "":
    commands = ("?", "enable", enablepw, "terminal pager 0", "no pager", "show conn", "disable", "exit")
else:
    commands = ("?", "enable", enableuser, enablepw, "terminal pager 0", "no pager", "show conn", "disable", "exit")
    
if sshversion == "1":
    sshversion = "ssh1"
else:
    sshversion = "ssh2"


print "Commands are ", commands

print "Polling device for first data set."

output = pix_scrape.scrape(targetip, user, linepw, commands, 22, sshversion)

print "Parsing first data set."
buf = conn_grep(output)
t1 = timeslice_from_data(buf)

print "Sleeping for ten seconds."
time.sleep(10)

print "Polling device for second data set."
output2 = pix_scrape.scrape(targetip, user, linepw, commands, 22, sshversion)

print "Parsing second data set."
buf2 = conn_grep(output2)
t2 = timeslice_from_data(buf2)

print "Redirecting stdout to file, back in a bit."
import sys
old_stdout = sys.stdout
sys.stdout = open(filename, 'a')

print_header()

print info_header

print """
<div class='main'>
<h3>Sortable Diff of the two "show conn" snapshots:</h3>
"""
print_table_header()

timeslice_from_diff(t2,t1).output()

print_table_footer()

print_footer()


rawfile = "%s%s_raw1.txt" % ("connflow", rawfilename)
sys.stdout = open(rawfile, 'a')
#print "<html><body"
#print """
#<div class='main'>
#<h3>Raw output from first snapshot:</h3>
#</div>"""
#print_sortable_raw(output)
#print "</body></html>"
for each in output:
    print each,

rawfile = "%s%s_raw2.txt" % ("connflow", rawfilename)
sys.stdout = open(rawfile, 'a')
#print "<html><body"
#print """
#<div class='main'>
#<h3>Raw output from second snapshot:</h3>
#</div>"""
#print_sortable_raw(output2)
#print "</body></html>"
for each in output:
    print each,

sys.stdout = old_stdout
print "File output:", filename
